/**
 * Locator main class
 *
 * Orchestrates LocatorSearch, LocatorList and LocatorMap
 *
 * @module package/quiqqer/locations/bin/frontend/controls/Locator/Locator
 * @author www.pcsg.de (Patrick Müller)
 */
define('package/quiqqer/locations/bin/frontend/controls/Locator/Locator', [

    'qui/controls/Control',
    'qui/controls/loader/Loader',
    'utils/Controls',

    'package/quiqqer/locations/bin/frontend/Locator',
    'package/quiqqer/geoutils/bin/classes/GeoPoint'

], function (QUIControl, QUILoader, QUIControlUtils, Locator, GeoPoint) {
    "use strict";

    return new Class({
        Extends: QUIControl,
        Type   : 'package/quiqqer/locations/bin/frontend/controls/Locator/Locator',

        Binds: [
            '$onImport',
            '$onSuggestSearch',
            '$onListEntryClick',
            '$onFilterChange',
            'reset',
            '$showResetBtn',
            '$hideResetBtn'
        ],

        options: {
            showresetbtn: false // show reset button on load
        },

        initialize: function (options) {
            this.parent(options);

            this.Loader = new QUILoader();

            this.$Search        = null;
            this.$List          = null;
            this.$Map           = null;
            this.$Filters       = null;
            this.$FilterData    = null;
            this.$geoIdentifier = null;
            this.$radius        = null;
            this.$updateViewBox = true;
            this.$ResetBtn      = null;

            this.addEvents({
                onImport: this.$onImport
            });
        },

        /**
         * Event: onImport
         */
        $onImport: function () {
            var Elm  = this.getElm();
            var self = this;

            this.Loader.inject(Elm);
            this.Loader.show();

            var promises = [],
                Search   = Elm.getElement('.quiqqer-locations-locator-search'),
                List     = Elm.getElement('.quiqqer-locations-locator-list'),
                Map      = Elm.getElement('.quiqqer-locations-locator-map'),
                Filters  = Elm.getElement('.quiqqer-locations-locator-filters');

            promises.push(Search ? QUIControlUtils.getControlByElement(Search) : Promise.resolve(false));
            promises.push(List ? QUIControlUtils.getControlByElement(List) : Promise.resolve(false));
            promises.push(Map ? QUIControlUtils.getControlByElement(Map) : Promise.resolve(false));
            promises.push(Filters ? QUIControlUtils.getControlByElement(Filters) : Promise.resolve(false));

            Promise.all(promises).then(function (result) {
                self.$Search  = result[0];
                self.$List    = result[1];
                self.$Map     = result[2];
                self.$Filters = result[3];

                // Register events
                if (self.$Search) {
                    self.$Search.addEvents({
                        onSearch: self.$onSuggestSearch
                    });
                }

                if (self.$List) {
                    self.$List.addEvents({
                        onEntryClick: self.$onListEntryClick
                    });
                }

                if (self.$Map) {
                    if (self.$Search) {
                        self.$Map.addEvents({
                            onMapLoaded: function () {
                                self.$Search.onMapLoaded();
                            }
                        });
                    }
                }

                if (self.$Filters) {
                    self.$Filters.addEvents({
                        onChange: self.$onFilterChange
                    });
                }

                self.Loader.hide();
            });

            // Reset button
            this.$ResetBtn = Elm.getElement('.quiqqer-locations-locator-search-reset');

            if (this.$ResetBtn) {
                this.$ResetBtn.addEvent('click', this.reset);

                if (this.getAttribute('showresetbtn')) {
                    this.$showResetBtn();
                }
            }
        },

        /**
         * Event: onSearch
         *
         * If the user clicks a location suggestion from LocationSearch
         *
         * @param {String} geoIdentifier
         * @param {Number} radius
         */
        $onSuggestSearch: function (geoIdentifier, radius) {
            this.$geoIdentifier = geoIdentifier;
            this.$radius        = radius;
            this.$updateViewBox = true;

            this.$search();
            this.$showResetBtn();
        },

        /**
         * Execute geo search
         */
        $search: function () {
            var self = this;

            if (!this.$geoIdentifier) {
                if (self.$List) {
                    self.$List.search(false, this.$radius, this.$FilterData);
                }

                if (self.$Map) {
                    self.$Map.search(false, this.$radius, this.$FilterData, this.$updateViewBox);
                }

                return;
            }

            this.Loader.show();

            Locator.getGeoPointByGeoIdentifier(this.$geoIdentifier).then(function (GeoPointData) {
                if (self.$List) {
                    self.$List.search(
                        new GeoPoint({
                            lat: GeoPointData.lat,
                            lng: GeoPointData.lng
                        }),
                        self.$radius,
                        self.$FilterData
                    );
                }

                if (self.$Map) {
                    self.$Map.search(
                        new GeoPoint({
                            lat: GeoPointData.lat,
                            lng: GeoPointData.lng
                        }),
                        self.$radius,
                        self.$FilterData
                    );
                }

                self.Loader.hide();
            });
        },

        /**
         * Reset user location search and all filter controls
         */
        reset: function () {
            var self = this;

            this.$geoIdentifier = false;
            this.$radius        = false;
            this.$updateViewBox = true;

            if (this.$Search) {
                this.$Search.reset();
            }

            if (this.$Filters) {
                this.$Filters.resetFilters().then(function () {
                    self.$FilterData = self.$Filters.getFilterData();
                    self.$search();
                });
            } else {
                this.$search();
            }

            this.$hideResetBtn();
        },

        /**
         * Event: onEntryClick
         *
         * If the user clicks a location entry in LocationList
         *
         * @param {Number} locationId
         */
        $onListEntryClick: function (locationId) {
            if (this.$Map) {
                this.$Map.openLocationInfoWindow(locationId);
            }
        },

        /**
         * Handle Filter changes
         *
         * @param {Object} FilterData
         */
        $onFilterChange: function (FilterData) {
            this.$FilterData    = FilterData;
            this.$updateViewBox = false;

            this.$search();
            this.$showResetBtn();
        },

        /**
         * Show "reset" button
         */
        $showResetBtn: function () {
            if (this.$ResetBtn) {
                this.$ResetBtn.setStyle('display', 'block');
            }
        },

        /**
         * Hide "reset" button
         */
        $hideResetBtn: function () {
            if (this.$ResetBtn) {
                this.$ResetBtn.setStyle('display', 'none');
            }
        }
    });
});
