/**
 * LocatorList main class
 *
 * Lists location results
 *
 * @module package/quiqqer/locations/bin/frontend/controls/Locator/LocatorList
 * @author www.pcsg.de (Patrick Müller)
 *
 * @event onEntryClick [locationId, this] - Fires if the users clicks a list entry
 */
define('package/quiqqer/locations/bin/frontend/controls/Locator/LocatorList', [

    'qui/QUI',
    'qui/controls/Control',
    'qui/controls/loader/Loader',

    'package/quiqqer/locations/bin/frontend/Locator'

], function (QUI, QUIControl, QUILoader, Locator) {
    "use strict";

    var lg = 'quiqqer/locations';

    return new Class({
        Extends: QUIControl,
        Type   : 'package/quiqqer/locations/bin/frontend/controls/Locator/LocatorList',

        Binds: [
            '$onImport',
            '$onPaginationChange'
        ],

        options: {
            showimages  : true,
            showdistance: true,
            list        : false     // HTML of initially shown list
        },

        initialize: function (options) {
            this.parent(options);

            this.Loader   = new QUILoader();
            this.$Content = null;

            // Search params
            this.$GeoCenter = null;
            this.$radius    = null;
            this.$page      = 1;

            this.$Pagination = null;

            this.addEvents({
                onImport: this.$onImport
            });
        },

        /**
         * Event: onImport
         */
        $onImport: function () {
            var self = this;
            var Elm  = this.getElm();

            Elm.set('html', '');

            this.$Content = Elm;
            this.Loader.inject(Elm);

            var listHtml = this.getAttribute('list');



            if (!listHtml) {
                this.search();
                return;
            }

            listHtml = JSON.decode(atob(listHtml));

            this.$Content.set('html', listHtml);
            this.$parseListElements();

            this.Loader.show();

            QUI.parse(this.$Content).then(function () {
                var PaginationElm = self.$Content.getElement(
                    '[data-qui="package/quiqqer/controls/bin/navigating/Pagination"]'
                );

                if (!PaginationElm) {
                    return;
                }

                self.$Pagination = QUI.Controls.getById(
                    PaginationElm.get('data-quiid')
                );

                self.$Pagination.addEvents({
                    change: self.$onPaginationChange
                });

                self.Loader.hide();
            });
        },

        /**
         * Perform a location search based on a center geo location
         *
         * @param {package/quiqqer/geoutils/bin/classes/GeoPoint} [GeoCenter]
         * @param {Number} [radius]
         * @param {Object} [FilterData]
         */
        search: function (GeoCenter, radius, FilterData) {
            var self     = this;
            var lat, lng = false;

            this.$GeoCenter = GeoCenter || null;
            this.$radius    = radius || null;

            if (this.$GeoCenter) {
                lat = this.$GeoCenter.lat();
                lng = this.$GeoCenter.lng();
            }

            this.Loader.show();

            Locator.getLocationList(
                lat,
                lng,
                this.$radius,
                this.$page,
                false,
                FilterData,
                this.getAttribute('showimages')
            ).then(function (Result) {
                self.$Content.set('html', Result);
                self.$parseListElements();

                QUI.parse(self.$Content).then(function () {
                    var PaginationElm = self.$Content.getElement(
                        '[data-qui="package/quiqqer/controls/bin/navigating/Pagination"]'
                    );

                    if (!PaginationElm) {
                        return;
                    }

                    self.$Pagination = QUI.Controls.getById(
                        PaginationElm.get('data-quiid')
                    );

                    self.$Pagination.addEvents({
                        change: self.$onPaginationChange
                    });

                    self.Loader.hide();
                });
            });
        },

        /**
         * Event: $onPaginationChange
         *
         * Execute paginated search if user changes the pagination sheet
         *
         * @param {Object} Pagination
         * @param {Object} Sheet
         * @param {Object} Query
         */
        $onPaginationChange: function (Pagination, Sheet, Query) {
            var self     = this,
                page     = Query.page,
                lat, lng = false;

            if (page == this.$page) {
                return;
            }

            if (this.$GeoCenter) {
                lat = this.$GeoCenter.lat();
                lng = this.$GeoCenter.lng();
            }

            this.Loader.show();

            Locator.getLocationList(
                lat,
                lng,
                this.$radius,
                Query.page,
                true
            ).then(function (Result) {
                self.$Content.getElement(
                    '.quiqqer-locations-locator-list-locations'
                ).set('html', Result);

                self.$parseListElements();

                self.$page = page;
                self.Loader.hide();
            });
        },

        /**
         * Parse all location entries and add click events
         */
        $parseListElements: function () {
            var listElements = this.$Content.getElements(
                '.quiqqer-locations-locator-list-locations-entry'
            );

            var self = this;

            listElements.addEvent('click', function (event) {
                var Elm = event.target;

                if (Elm.className !== 'quiqqer-locations-locator-list-locations-entry') {
                    Elm = Elm.getParent('.quiqqer-locations-locator-list-locations-entry');
                }

                self.fireEvent('entryClick', [Elm.get('data-id'), self]);
            });
        }
    });
});
