/**
 * LocatorSearch main class
 *
 * Suggest search for user location
 *
 * @module package/quiqqer/locations/bin/frontend/controls/Locator/LocatorSearch
 * @author www.pcsg.de (Patrick Müller)
 *
 * @event onSearch [geoIdentifier, radius, this] - Fires if the user clicks a search suggestion
 */
define('package/quiqqer/locations/bin/frontend/controls/Locator/LocatorSearch', [

    'qui/controls/Control',
    'qui/controls/loader/Loader',
    'qui/controls/input/Suggest',
    'qui/utils/Functions',

    'Locale',

    'package/quiqqer/geoutils/bin/Geo',
    'package/quiqqer/geoutils/bin/classes/GeoPoint',

    'package/quiqqer/locations/bin/frontend/Locator',

    'css!package/quiqqer/locations/bin/frontend/controls/Locator/LocatorSearch.css'

], function (QUIControl, QUILoader, QUISuggest, QUIFunctionUtils, QUILocale, GeoApi, GeoPoint, Locator) {
    "use strict";

    var lg = 'quiqqer/locations';

    return new Class({
        Extends: QUIControl,
        Type   : 'package/quiqqer/locations/bin/frontend/controls/Locator/LocatorSearch',

        Binds: [
            '$onImport',
            '$search',
            '$onSearch',
            'reset',
            '$detectUserLocation',
            'onMapLoaded'
        ],

        options: {
            radius        : false,    // initial value for radius
            searchstring  : false,    // initial value for location search input
            autodetect    : false,    // auto-detect user location on load
            defaultcountry: false
        },

        initialize: function (options) {
            this.parent(options);

            this.Loader              = new QUILoader();
            this.$Suggest            = null;
            this.$CountrySelect      = null;
            this.$RadiusSelect       = null;
            this.$radiusDefault      = 25;
            this.$geoIdentifier      = null;
            this.$geoLocatingAllowed = false;

            this.addEvents({
                onImport: this.$onImport
            });
        },

        /**
         * Event: onImport
         */
        $onImport: function () {
            var self = this;
            var Elm  = this.getElm();

            this.Loader.inject(Elm);

            var onSuggestChange = QUIFunctionUtils.debounce(self.$search, 500);

            this.$Suggest = new QUISuggest({
                autosearch: false,
                events    : {
                    onChange: onSuggestChange,
                    onSelect:
                        function (data) {
                            var SuggestInput = self.$Suggest.getInput();

                            if (SuggestInput.value.trim() === '') {
                                self.$geoIdentifier = null;
                                self.$Suggest.clearOptions();
                            } else {
                                self.$geoIdentifier         = data.value;
                                self.$RadiusSelect.disabled = false;
                            }

                            self.$onSearch();
                        }
                }
            }).imports(
                Elm.getElement('input.quiqqer-locations-locator-search-input')
            );

            if (this.getAttribute('searchstring')) {
                this.$Suggest.getInput().value = this.getAttribute('searchstring');
            }

            this.$Suggest.getElm().addClass('quiqqer-locations-locator-search-suggest');

            this.$RadiusSelect = Elm.getElement(
                'select.quiqqer-locations-locator-search-radius'
            );

            if (this.getAttribute('radius')) {
                this.$RadiusSelect.value = this.getAttribute('radius');
            }

            this.$radiusDefault = this.$RadiusSelect.value;

            this.$RadiusSelect.addEvents({
                change: this.$onSearch
            });

            this.$CountrySelect = Elm.getElement('select.quiqqer-locations-locator-search-country');

            // Determine user location via Geolocation
            var GeoLocateBtn = Elm.getElement('.quiqqer-locations-locator-search-geolocate');

            if (!navigator.geolocation) {
                return;
            }

            this.$geoLocatingAllowed = true;

            GeoLocateBtn.setStyle('display', 'inline');

            GeoLocateBtn.addEvent('click', function (event) {
                event.stop();
                self.$detectUserLocation();
            });
        },

        /**
         * Perform actions if the Locator Map is loaded
         */
        onMapLoaded: function () {
            if (this.$geoLocatingAllowed && this.getAttribute('autodetect')) {
                this.$detectUserLocation();
            }
        },

        /**
         * Auto-detect user location via browser geolocation API
         */
        $detectUserLocation: function () {
            var self = this;

            this.Loader.show();

            navigator.geolocation.getCurrentPosition(
                function (Position) {
                    Locator.getUserGeoLocationData(
                        Position.coords.latitude,
                        Position.coords.longitude
                    ).then(function (LocationData) {
                        self.Loader.hide();

                        if (!LocationData) {
                            return;
                        }

                        self.$Suggest.getInput().value = LocationData.title;

                        if (self.$CountrySelect) {
                            self.$CountrySelect.value = LocationData.country.toUpperCase();
                        }

                        self.$geoIdentifier         = LocationData.geoIdentifier;
                        self.$RadiusSelect.disabled = false;

                        self.$onSearch();
                    });
                },
                function () {
                    self.Loader.hide();
                }
            );
        },

        /**
         * Perform location search based on user input and show suggestions
         */
        $search: function () {
            var self        = this,
                searchValue = this.$Suggest.getInput().value.trim(),
                country     = this.getAttribute('defaultcountry');

            if (this.$CountrySelect) {
                country = this.$CountrySelect.value.trim();
            }

            this.$geoIdentifier = null;

            if (searchValue.length < 3) {
                return;
            }

            this.$Suggest.getInput().value = searchValue;
            this.$Suggest.showLoader();

            GeoApi.getSuggestionListBySearchString(searchValue, country).then(function (results) {
                self.$Suggest.clearOptions();

                if (!results.length) {
                    var SuggestInput = self.$Suggest.getInput();

                    var NoResults = new Element('div', {
                        'class': 'quiqqer-locations-locator-no-results',
                        html   : QUILocale.get(lg, 'controls.Locator.LocatorSearch.no_results'),
                        styles : {
                            position: 'fixed',
                            top     : SuggestInput.getPosition().y + SuggestInput.getSize().y
                        }
                    }).inject(self.$Suggest.getInput(), 'after');

                    self.$Suggest.hideLoader();

                    (function () {
                        NoResults.destroy();
                    }).delay(2000);
                    return;
                }

                for (var i = 0, len = results.length; i < len; i++) {
                    var suggestion = results[i];

                    self.$Suggest.addOption(
                        suggestion.description,
                        suggestion.identifier
                    );
                }

                self.$Suggest.refreshSuggest();
                self.$Suggest.hideLoader();
                self.$Suggest.showSuggest();
            });
        },

        /**
         * Event: onSearch
         *
         * If the user clicks a place suggestion from the list or changes the radius
         */
        $onSearch: function () {
            this.fireEvent('search', [this.$geoIdentifier, this.$RadiusSelect.value, this]);
        },

        /**
         * Reset search settings
         */
        reset: function () {
            this.$RadiusSelect.value    = this.$radiusDefault;
            this.$RadiusSelect.disabled = true;
            this.$geoIdentifier         = null;

            this.$Suggest.getInput().value = '';
            this.$Suggest.clearOptions();
        }
    });
});
