<?php

namespace QUI\Locations\Attributes\Types;

use QUI;
use QUI\Locations\Attributes\AbstractAttribute;

/**
 * Class Coordinates
 *
 * Contains coordinates for a location
 *
 * @package QUI\Locations
 */
class Coordinates extends AbstractAttribute
{
    /**
     * Validate the value of an attribute
     *
     * @param string|array $value - The input value (from the frontend)
     * @return bool - true, if value is valid; otherwise an exception is thrown
     *
     * @throws QUI\Locations\Exception
     */
    public function validate($value)
    {
        if (empty($value)) {
            return true;
        }

        if (is_string($value)) {
            $value = json_decode($value, true);

            if (json_last_error() !== JSON_ERROR_NONE) {
                throw new QUI\Locations\Exception([
                    'quiqqer/locations',
                    'exception.attributes.types.datatype.json.error'
                ]);
            }
        }

        if (!is_array($value)) {
            throw new QUI\Locations\Exception([
                'quiqqer/locations',
                'exception.attributes.types.datatype.array.expected'
            ]);
        }

        if (!isset($value['coordinates'])
            || !isset($value['coordinates']['lat'])
            || !isset($value['coordinates']['lng'])
            || !isset($value['location'])
        ) {
            throw new QUI\Locations\Exception([
                'quiqqer/locations',
                'exception.attributes.types.datatype.wrong'
            ]);
        }

        return true;
    }

    /**
     * Get the value that is used in the search cache table for this attribute
     *
     * @param QUI\Locale $Locale (optional) - Value depending on specific Locale (language);
     * if omitted QUI\Locations\Handler\Locations::getLocale is used
     * @return string - Database value
     */
    public function getSearchCacheValue($Locale = null)
    {
        if (empty($this->value)) {
            return null;
        }

        $value       = json_decode($this->value, true);
        $coordinates = $value['coordinates'];

        $data = [];

        if (!empty($coordinates['lat'])) {
            $data[] = $coordinates['lat'];
        }

        if (!empty($coordinates['lng'])) {
            $data[] = $coordinates['lng'];
        }

        return implode(',', $data);
    }

    /**
     * Get the value that is used in the search cache table for this attribute
     *
     * @param QUI\Locale $Locale (optional) - Value depending on specific Locale (language);
     * if omitted QUI\Locations\Handler\Locations::getLocale is used
     * @return mixed - View value
     */
    public function getViewValue($Locale = null)
    {
        if (empty($this->value)) {
            return '';
        }

        $value       = json_decode($this->value, true);
        $coordinates = $value['coordinates'];

        return [
            'lat' => !empty($coordinates['lat']) ? $coordinates['lat'] : false,
            'lng' => !empty($coordinates['lng']) ? $coordinates['lng'] : false,
        ];
    }
}
