<?php

namespace QUI\Locations\Attributes\Types;

use QUI;
use QUI\Locations\Attributes\AbstractAttribute;
use QUI\Locations\Handler\Locations as LocationsHandler;

/**
 * Class MultiLang
 *
 * A multi-lingual attribute
 *
 * @package QUI\Locations
 */
class InputMultiLang extends AbstractAttribute
{
    /**
     * Validate the value of an attribute
     *
     * @param string|array $value - The input value (from the frontend)
     * @return bool - true, if value is valid; otherwise an exception is thrown
     *
     * @throws QUI\Locations\Exception
     */
    public function validate($value)
    {
        if (is_string($value)) {
            $value = json_decode($value, true);

            if (json_last_error() !== JSON_ERROR_NONE) {
                throw new QUI\Locations\Exception([
                    'quiqqer/locations',
                    'exception.attributes.types.datatype.json.error'
                ]);
            }
        }

        if (!is_array($value)) {
            throw new QUI\Locations\Exception([
                'quiqqer/locations',
                'exception.attributes.types.datatype.array.expected'
            ]);
        }

        $langs = QUI::availableLanguages();

        foreach ($value as $lang => $langValue) {
            if (!is_string($lang)) {
                throw new QUI\Locations\Exception([
                    'quiqqer/locations',
                    'exception.attributes.types.datatype.wrong'
                ]);
            }

            if (!in_array($lang, $langs)) {
                throw new QUI\Locations\Exception([
                    'quiqqer/locations',
                    'exception.attributes.types.inputmultilang.lang.not.found',
                    [
                        'unknownLang' => $lang
                    ]
                ]);
            }
        }

        return true;
    }

    /**
     * Get the value that is used in the search cache table for this attribute
     *
     * @param QUI\Locale $Locale (optional) - Value depending on specific Locale (language);
     * if omitted QUI\Locations\Handler\Locations::getLocale is used
     * @return string - Database value
     */
    public function getSearchCacheValue($Locale = null)
    {
        if (is_null($Locale)) {
            $Locale = LocationsHandler::getLocale();
        }

        $lang   = $Locale->getCurrent();
        $values = json_decode($this->value, true);

        if (isset($values[$lang])) {
            return $values[$lang];
        }

        return '';
    }

    /**
     * Get the value that is used in the search cache table for this attribute
     *
     * @param QUI\Locale $Locale (optional) - Value depending on specific Locale (language);
     * if omitted QUI\Locations\Handler\Locations::getLocale is used
     * @return string - Database value
     */
    public function getViewValue($Locale = null)
    {
        if (is_null($Locale)) {
            $Locale = LocationsHandler::getLocale();
        }

        $lang   = $Locale->getCurrent();
        $values = json_decode($this->value, true);

        if (isset($values[$lang])) {
            return $values[$lang];
        }

        return '';
    }
}
