<?php

namespace QUI\Locations\Attributes\Types;

use QUI;
use QUI\Locations\Attributes\AbstractAttribute;
use QUI\Locations\Utils as LocationUtils;

/**
 * Class OpeningHours
 *
 * Attribute that contains regular opening hours of a location
 *
 * @package QUI\Locations
 */
class OpeningHours extends AbstractAttribute
{
    /**
     * Validate the value of an attribute
     *
     * @param string|array $value - The input value (from the frontend)
     * @return bool - true, if value is valid; otherwise an exception is thrown
     *
     * @throws QUI\Locations\Exception
     */
    public function validate($value)
    {
        if (empty($value)) {
            return true;
        }

        if (is_string($value)) {
            $value = json_decode($value, true);

            if (json_last_error() !== JSON_ERROR_NONE) {
                throw new QUI\Locations\Exception([
                    'quiqqer/locations',
                    'exception.attributes.types.datatype.json.error'
                ]);
            }
        }

        if (!is_array($value)) {
            throw new QUI\Locations\Exception([
                'quiqqer/locations',
                'exception.attributes.types.datatype.array.expected'
            ]);
        }

        foreach ($value as $data) {
            if (!isset($data['closed24'])
                || !isset($data['open24'])
                || !isset($data['range'])
                || !isset($data['from'])
                || !isset($data['to'])
            ) {
                throw new QUI\Locations\Exception([
                    'quiqqer/locations',
                    'exception.attributes.types.datatype.wrong'
                ]);
            }

            // check range
            switch ($data['range']) {
                case 'motofr':
                case 'motosa':
                case 'sasu':
                case 'mo':
                case 'tu':
                case 'we':
                case 'th':
                case 'fr':
                case 'sa':
                case 'su':
                case 'everyday':
                    break;

                default:
                    throw new QUI\Locations\Exception([
                        'quiqqer/locations',
                        'exception.attributes.types.datatype.wrong'
                    ]);
            }

            if ($data['closed24'] && $data['open24']) {
                throw new QUI\Locations\Exception([
                    'quiqqer/locations',
                    'exception.attributes.types.openinghours.cannot.be.opened.and.closed'
                ]);
            }

            // check time entries
            if (mb_strpos($data['from'], ':') !== false
                || mb_strpos($data['to'], ':') !== false
            ) {
                throw new QUI\Locations\Exception([
                    'quiqqer/locations',
                    'exception.attributes.types.datatype.wrong'
                ]);
            }

            $from = str_replace(':', '', $data['from']);
            $to   = str_replace(':', '', $data['to']);

            if (preg_match('#\D#i', $from) !== 0
                || preg_match('#\D#i', $to) !== 0
            ) {
                throw new QUI\Locations\Exception([
                    'quiqqer/locations',
                    'exception.attributes.types.datatype.wrong'
                ]);
            }

            $fromHour   = mb_substr($from, 0, 2);
            $fromMinute = mb_substr($from, 2);

            $toHour   = mb_substr($to, 0, 2);
            $toMinute = mb_substr($to, 2);

            if (mb_strlen($fromHour) > 2
                || mb_strlen($toHour) > 2
                || $fromHour > 23
                || $fromHour < 0
                || $toHour > 23
                || $toHour < 0
            ) {
                throw new QUI\Locations\Exception([
                    'quiqqer/locations',
                    'exception.attributes.types.openinghours.hour.out.of.range'
                ]);
            }

            if (mb_strlen($toMinute) > 2
                || mb_strlen($fromMinute) > 2
                || $fromMinute > 59
                || $fromMinute < 0
                || $toMinute > 59
                || $toMinute < 0
            ) {
                throw new QUI\Locations\Exception([
                    'quiqqer/locations',
                    'exception.attributes.types.openinghours.minute.out.of.range'
                ]);
            }

            if ((int)($fromHour.$fromMinute) > (int)($toHour.$toMinute)) {
                throw new QUI\Locations\Exception([
                    'quiqqer/locations',
                    'exception.attributes.types.openinghours.open.before.closed'
                ]);
            }
        }

        return true;
    }

    /**
     * Get the value that is used in the search cache table for this attribute
     *
     * @param QUI\Locale $Locale (optional) - Value depending on specific Locale (language);
     * if omitted QUI\Locations\Handler\Locations::getLocale is used
     * @return string - Database value
     */
    public function getSearchCacheValue($Locale = null)
    {
        if (empty($this->value)) {
            return null;
        }

        return $this->value;
    }

    /**
     * Get the value that is used in the search cache table for this attribute
     *
     * @param QUI\Locale $Locale (optional) - Value depending on specific Locale (language);
     * if omitted QUI\Locations\Handler\Locations::getLocale is used
     * @return string - Database value
     */
    public function getViewValue($Locale = null)
    {
        if (empty($this->value)) {
            return $this->value;
        }

        // determine if location is currently open
        $value = json_decode($this->value, true);

        if (empty($value)) {
            return $value;
        }

        $open         = false;
        $openingLines = [];

        foreach ($value as $k => $entry) {
            if (empty($entry['closed24'])
                && empty($entry['open24'])
                && empty($entry['from'])
                && empty($entry['to'])) {
                continue;
            }

            if (LocationUtils::isOpen($entry)) {
                $open = true;
            }

            $openingLines[] = LocationUtils::convertOpeningDataToText($entry);
        }

        $value['openNow']      = $open;
        $value['openingLines'] = $openingLines;

        return $value;
    }
}
