<?php

namespace QUI\Locations\Controls\Lists;

use QUI;
use QUI\LocationCategories\Handler as CategoryHandler;
use QUI\Locations\Handler\Locations;
use QUI\Locations\Locator\Handler as LocatorHandler;

/**
 * Class LocationList
 *
 * Listing of locations with pagination
 */
class LocationList extends QUI\Control
{
    const SORT_EDIT_DATE_DESC   = 'edit_date_desc';
    const SORT_CREATE_DATE_DESC = 'create_date_desc';

    /**
     * constructor
     *
     * @param array $attributes
     */
    public function __construct($attributes = [])
    {
        // default options
        $this->setAttributes([
            'class'                      => 'qui-control-list',
            'limit'                      => 5,
            'page'                       => 1,
            'sort'                       => self::SORT_EDIT_DATE_DESC,
            'categoryId'                 => false,     // restrict to locations of given category
            'showSheets'                 => true,
            'showImages'                 => true,
            'showShort'                  => true,
            'showHeader'                 => true,
            'showContent'                => false,
            'showCity'                   => true,
            'showMoreBtn'                => true,
            'Site'                       => false,
            'parentInputList'            => false,
            'itemtype'                   => 'http://schema.org/ItemList',
            'child-itemtype'             => 'https://schema.org/ListItem',
            'child-itemprop'             => 'itemListElement',
            // layout / design
            'display'                    => 'default',
            // Custom children template (path to html file); overwrites "display"
            'displayTemplate'            => false,
            // Custom children template css (path to css file); overwrites "display"
            'displayCss'                 => false,
            'nodeName'                   => 'section',
            // list of sites to display,
            'children'                   => false,
            'showPagination'             => false,
            'locationCountOnEmptySearch' => 0,         // number of locations that are shown if search results is empty
            'locationIds'                => false,     // Array of Location IDs that should be displayed
            'listHash'                   => false,     // Hash that identifies a specific set of location IDs
            'emptyText'                  => false      // Custom text for empty location list
        ]);

        parent::__construct($attributes);
    }

    /**
     * Return the inner body of the element
     * Can be overwritten
     *
     * @return String
     *
     * @throws QUI\Exception
     */
    public function getBody()
    {
        try {
            $Engine = QUI::getTemplateManager()->getEngine();
        } catch (QUI\Exception $Exception) {
            QUI\System\Log::writeException($Exception);

            return '';
        }

        $limit = $this->getAttribute('limit');
        $start = ((int)$this->getAttribute('page') - 1) * $limit;

        if (!$limit) {
            $limit = 6;
        }

        if ($this->getAttribute('showSheets')) {
            if (isset($_REQUEST['sheet'])) {
                $start = ((int)$_REQUEST['sheet'] - 1) * $limit;
            }

            $Engine->assign('showSheets', true);
        }

        $locationIds = $this->getLocationIds();

        if ($this->getAttribute('listHash')) {
            $this->setJavaScriptControlOption('listhash', $this->getAttribute('listHash'));
        }

        // sheets
        $sheets = \ceil(\count($locationIds) / $limit);

        // apply limit & load Location objects
//        if ($this->getAttribute('display') !== 'Slider') {
        $locationIds = \array_slice($locationIds, $start, $limit);
//        }

        $locations = [];
        $Site      = $this->getSite();
        $Project   = $Site->getProject();

        // Determine if no original locations are shown and if so -> show fallback list
        $noResults = \is_array($locationIds) && empty($locationIds);

        if ($noResults && $this->getAttribute('locationCountOnEmptySearch')) {
            $locationIds = $this->getAllLocationIds();
            $locationIds = \array_slice($locationIds, 0, $this->getAttribute('locationCountOnEmptySearch'));
        }

        foreach ($locationIds as $locationId) {
            try {
                $Location     = Locations::getLocation($locationId);
                $locationData = $Location->getView()->toArray();

                // Get Location URL
                $locationData['url'] = $Location->getUrl($Project);

                $locations[] = $locationData;
            } catch (\Exception $Exception) {
                QUI\System\Log::writeException($Exception);
            }
        }

        // Pagination
        $Pagination = false;

        if ($this->getAttribute('showPagination')) {
            $Pagination = new QUI\Controls\Navigating\Pagination([
                'useAjax' => true,
                'Site'    => $Site
            ]);

            $Pagination->setAttribute('sheets', $sheets);
            $Pagination->setAttribute('limit', $limit);

            $Pagination->loadFromRequest();
        }

        // Sort
        switch ($this->getAttribute('sort')) {
            case self::SORT_CREATE_DATE_DESC:
            case self::SORT_EDIT_DATE_DESC:
                $this->setJavaScriptControlOption('sort', $this->getAttribute('sort'));
                break;
        }

        // Determine message for empty list
        if ($this->getAttribute('emptyText')) {
            $emptyText = $this->getAttribute('emptyText');
        } else {
            $categoryTitle = $this->getCategoryTitle();

            if ($categoryTitle) {
                $emptyText = QUI::getLocale()->get(
                    'quiqqer/locations',
                    'LocationList.empty_category',
                    [
                        'categoryTitle' => $categoryTitle
                    ]
                );
            } else {
                $emptyText = QUI::getLocale()->get(
                    'quiqqer/locations',
                    'LocationList.empty'
                );
            }
        }

        if ($noResults && $this->getAttribute('locationCountOnEmptySearch')) {
            $emptyText .= ' ';
            $emptyText .= QUI::getLocale()->get(
                'quiqqer/locations',
                'LocationList.empty_default_suffix'
            );
        }

        // Assign template variables
        $Engine->assign([
            'noResults'   => $noResults,
            'this'        => $this,
            'Site'        => $this->getSite(),
            'Project'     => $this->getProject(),
            'sheets'      => $sheets,
            'locations'   => $locations,
            'Pagination'  => $Pagination,
            'Events'      => $this->Events,
            'LocatorSite' => LocatorHandler::getLocatorSite(),
            'emptyText'   => $emptyText
        ]);

        // Load custom template (if set)
        if ($this->getAttribute('displayTemplate')
            && \file_exists($this->getAttribute('displayTemplate'))
        ) {
            if ($this->getAttribute('displayCss')
                && \file_exists($this->getAttribute('displayCss'))
            ) {
                $this->addCSSFile($this->getAttribute('displayCss'));
            }

            return $Engine->fetch($this->getAttribute('displayTemplate'));
        }

        // Load display type
        $display   = $this->getAttribute('display');
        $jsControl = 'package/quiqqer/locations/bin/frontend/controls/LocationList';

        switch ($display) {
            case 'Slider':
                $jsControl = 'package/quiqqer/locations/bin/frontend/controls/LocationList.Slider';
                $css       = \dirname(__FILE__).'/LocationList.Slider.css';
                $template  = \dirname(__FILE__).'/LocationList.Slider.html';
                $height    = 300;

                if ($this->getAttribute('height')) {
                    $height = $this->getAttribute('height');
                    $this->setAttribute('height', false);
                }
                $Engine->assign('height', $height);
                break;
            case 'GalleryList2Col':
            case 'GalleryList3Col':
            case 'GalleryList4Col':
                $css      = \dirname(__FILE__).'/LocationList.'.$display.'.css';
                $template = \dirname(__FILE__).'/LocationList.'.$display.'.html';
                break;

            default:
                $css      = \dirname(__FILE__).'/LocationList.List.css';
                $template = \dirname(__FILE__).'/LocationList.List.html';
        }

        $this->setJavaScriptControl($jsControl);
        $this->setJavaScriptControlOption('categoryid', (int)$this->getAttribute('categoryId'));

        // Load default template and CSS
        $this->addCSSFile($css);

        return $Engine->fetch($template);
    }

    /**
     * Get title of the location category that is operated with
     *
     * @return string|false - Title or false if category could not be determined
     */
    protected function getCategoryTitle()
    {
        $Site = $this->getSite();

        if (!empty($Site)) {
            return $Site->getAttribute('title');
        }

        $categoryId = $this->getAttribute('categoryId');

        if (empty($categoryId) || !QUI::getPackageManager()->isInstalled('quiqqer/locationcategories')) {
            return false;
        }

        try {
            return CategoryHandler::getCategory($categoryId)->getTitle();
        } catch (\Exception $Exception) {
            QUI\System\Log::writeException($Exception);
            return false;
        }
    }

    /**
     * Get all location IDs the location search shall be restricted to
     *
     * @return int[]|false - List of location IDs or false in no restriction
     */
    protected function getRestrictToLocationIds()
    {
        // Filter locations by category?
        $categoryId            = $this->getAttribute('categoryId');
        $restrictToLocationIds = false;

        if (!empty($categoryId) && QUI::getPackageManager()->isInstalled('quiqqer/locationcategories')) {
            try {
                $restrictToLocationIds = CategoryHandler::getCategory($categoryId)->getLocationIds(false, true);
            } catch (\Exception $Exception) {
                QUI\System\Log::writeException($Exception);
            }
        }

        return $restrictToLocationIds;
    }

    /**
     * @return int[]
     */
    protected function getLocationIds()
    {
        // First: Location IDs from attribute
        $locationIds           = $this->getAttribute('locationIds');
        $restrictToLocationIds = $this->getRestrictToLocationIds();

        if ($locationIds !== false) {
            // Apply filter
            if ($restrictToLocationIds !== false) {
                $locationIds = \array_intersect($locationIds, $restrictToLocationIds);
            }

            $locationIdsJson = \json_encode($locationIds);
            $listHash        = \md5($locationIdsJson);
            QUI::getSession()->set('quiqqer-locations-list-'.$listHash, \json_encode($locationIds));

            $this->setAttribute('listHash', $listHash);

            return $locationIds;
        }

        // Second: Location IDs by special list hash (these are not filtered)!
        $listHash = $this->getAttribute('listHash');

        if (!empty($listHash) && \is_string($listHash)) {
            $locationIds = QUI::getSession()->get('quiqqer-locations-list-'.$listHash);

            if (!empty($locationIds)) {
                return \json_decode($locationIds, true);
            }
        }

        return $this->getAllLocationIds();
    }

    /**
     * Get all location IDs (may be restricted by category)
     *
     * @return int[]
     */
    protected function getAllLocationIds()
    {
        $restrictToLocationIds = $this->getRestrictToLocationIds();

        $where = [
            'active' => 1
        ];

        if ($restrictToLocationIds !== false) {
            if (empty($restrictToLocationIds)) {
                return [];
            }

            $where['id'] = [
                'type'  => 'IN',
                'value' => $restrictToLocationIds
            ];
        }

        // ORDER BY
        switch ($this->getAttribute('sort')) {
            case self::SORT_CREATE_DATE_DESC:
                $order = 'c_date DESC';
                break;

            default:
                $order = 'e_date DESC';
        }

        try {
            $result = QUI::getDataBase()->fetch([
                'select' => ['id'],
                'from'   => Locations::getLocationsTable(),
                'where'  => $where,
                'order'  => $order
            ]);
        } catch (\Exception $Exception) {
            QUI\System\Log::writeException($Exception);
            return [];
        }

        return \array_column($result, 'id');
    }

    /**
     * @return mixed|QUI\Projects\Site|false
     */
    protected function getSite()
    {
        if ($this->getAttribute('Site')) {
            return $this->getAttribute('Site');
        }

        try {
            $Site = QUI::getRewrite()->getSite();
        } catch (\Exception $Exception) {
            QUI\System\Log::writeException($Exception);
            return false;
        }

        $this->setAttribute('Site', $Site);

        return $Site;
    }
}
