<?php

namespace QUI\Locations\Controls;

use QUI;
use QUI\Countries\Manager as Countries;
use QUI\Geo\Utils\Handler\Modules as GeoModules;
use QUI\Locations\Locator\Handler as LocatorHandler;
use QUI\Locations\Handler\Locations;

/**
 * Class Locator
 *
 * Main control for Locator
 */
class Locator extends QUI\Control
{
    /**
     * constructor
     *
     * @param array $attributes
     */
    public function __construct($attributes = [])
    {
        $this->setAttributes([
            'Site'                 => false,
            // show these locations initially (map and/or list)
            'locationIds'          => false,
            // set initial filter data for filter controls
            'filterData'           => [],
            // initial search radius
            'radius'               => false,
            // search string that is initially put in search box (if user location search is enabled)
            'locationSearchString' => false,
            // template: filterSlideOut | ResultListBottom | 3Columns
            'display'              => '3Columns',
            // show location main image in location list
            'locationImageList'    => false
        ]);

        $this->addCSSClass('quiqqer-locations-locator');
        $this->setJavaScriptControl('package/quiqqer/locations/bin/frontend/controls/Locator/Locator');

        parent::__construct($attributes);
    }

    /**
     * (non-PHPdoc)
     *
     * @throws QUI\Exception
     * @see \QUI\Control::create()
     *
     */
    public function getBody()
    {
        $Engine  = QUI::getTemplateManager()->getEngine();
        $Site    = $this->getSite();
        $Project = $Site->getProject();
//        $url     = rtrim($Project->get(1)->getUrlRewrittenWithHost(), '/');

        /**
         * Locator LIST
         */
        $listSettings = [
            'showimages'   => $Site->getAttribute('quiqqer.locations.locator.list.show_images'),
            'showdistance' => $Site->getAttribute('quiqqer.locations.locator.list.show_distance')
        ];

        $listContent = $this->getListContent(); // initial HTML content for Locator List

        if ($listContent) {
            $listSettings['list'] = \base64_encode(\json_encode($listContent));
        }

        /**
         * Locator MAP
         */
        $mapType = $Site->getAttribute('quiqqer.locations.locator.map.defaultMapType');

        if ($mapType === 'default') {
            $mapType = LocatorHandler::getLocatorSetting('defaultMapType');
        }

        $mapSettings = [
            'layershowimages'    => $Site->getAttribute('quiqqer.locations.locator.map.info_layer.show_images'),
            'maptype'            => $mapType,
            'center'             => \base64_encode(\json_encode(LocatorHandler::getDefaultMapViewCenter($Site))),
            'showonemptyresults' => LocatorHandler::getLocatorSetting('showOnEmptySearchResults')
        ];

        $mapLocations = $this->getMapLocationList();

        if ($mapLocations !== false) {
            $mapSettings['locations'] = \base64_encode(\json_encode($mapLocations));

            // Determine viewbox for pre-defined locations
            if (!empty($mapLocations)) {
                $lat = array_column($mapLocations, 'lat');
                $lng = array_column($mapLocations, 'lng');

                sort($lat);
                sort($lng);

                $viewBox = [
                    'northLat' => $lat[count($lat) - 1],
                    'southLat' => $lat[0],
                    'eastLng'  => $lng[count($lng) - 1],
                    'westLng'  => $lng[0]
                ];

                $mapSettings['viewbox'] = \base64_encode(\json_encode($viewBox));
            }
        }

        // Map marker icon
        $mapMarkerIcon     = $Site->getAttribute('quiqqer.locations.locator.map.marker_icon');
        $infoWindowOffsetY = (int)$Site->getAttribute('quiqqer.locations.locator.map.marker_info_window_offset');

//        if (!empty($mapMarkerIcon)) {
//            try {
//                $Image                     = QUI\Projects\Media\Utils::getImageByUrl($mapMarkerIcon);
//                $infoWindowOffsetY         = $Image->getHeight();
//                $mapSettings['markericon'] = $url.$Image->getSizeCacheUrl(true);
//            } catch (\Exception $Exception) {
//                QUI\System\Log::writeException($Exception);
//            }
//        }

        $mapSettings['infowindowoffsety'] = $infoWindowOffsetY;

        /**
         * Locator USER LOCATION FILTER/SEARCH
         */
        $userLocationSettings = [];

        if ($this->getAttribute('radius')) {
            $userLocationSettings['radius'] = $this->getAttribute('radius');
        }

        if ($this->getAttribute('locationSearchString')) {
            $userLocationSettings['searchstring'] = $this->getAttribute('locationSearchString');
            $this->setJavaScriptControlOption('showresetbtn', true);
        }

        $autoDetectUserLocation = $Site->getAttribute('quiqqer.locations.locator.user_location_filter.auto_geolocate');

        if (!empty($autoDetectUserLocation)) {
            $userLocationSettings['autodetect'] = true;
        }

        // Country select
        $showCountrySelect = false;

        if (!empty($Site->getAttribute('quiqqer.locations.locator.user_location_filter.show_country'))) {
            $showCountrySelect = true;
        }

        if ($showCountrySelect) {
            // Try to autoamtically determine use country
            $UserCountry = QUI::getUserBySession()->getCountry();
            $userCountry = false;

            if (empty($UserCountry)) {
                if (Countries::existsCountryCode(QUI::getLocale()->getCurrent())) {
                    $userCountry = \mb_strtoupper(QUI::getLocale()->getCurrent());
                }
            } else {
                $userCountry = \mb_strtoupper($UserCountry->getCode());
            }

            // Fallback - Use default country from Site setting
            $defaultCountry = $Site->getAttribute('quiqqer.locations.locator.user_location_filter.country_default');

            if (!empty($defaultCountry)) {
                $userCountry = $defaultCountry;
            }

            if ($userCountry) {
                $userLocationSettings['defaultcountry'] = \mb_strtoupper($userCountry);
            } else {
                // Fallback fallback - use Germany
                $userLocationSettings['defaultcountry'] = 'DE';
            }
        }

        // Radius options
        $radiusOptions = LocatorHandler::getLocatorSetting('radius_options');

        if (empty($radiusOptions)) {
            $radiusOptions = [10, 25, 50, 100, 200];
        } else {
            $radiusOptions = \explode(",", $radiusOptions);
        }

        $radiusDefault = $Site->getAttribute('quiqqer.locations.locator.user_location_filter.radius_default');

        if (empty($radiusDefault)) {
            $radiusDefault = LocatorHandler::getLocatorSetting('radius_default');
        }

        if (empty($radiusDefault)) {
            $radiusDefault = 50;
        }

        // Determine how locations are presented
        $showList               = boolval($Site->getAttribute('quiqqer.locations.locator.list.show'));
        $showMap                = boolval($Site->getAttribute('quiqqer.locations.locator.map.show'));
        $showUserLocationFilter = boolval($Site->getAttribute('quiqqer.locations.locator.user_location_filter.show'));

        if (!GeoModules::getGeoModule()->hasMapSupport()) {
            $showMap = false;
        }

        if (!$showList && !$showMap) {
            $showList = true;
        }

        /**
         * Locator FILTERS
         */
        $showFilters    = true; // @todo optional, take from $Site attribute
        $filterControls = [];

        if ($showFilters) {
            $filterControls = LocatorHandler::getFilterControls();
            $filterData     = $this->getAttribute('filterData');

            foreach ($filterControls as $idHash => $FilterControl) {
                $FilterControl->addCSSClass('quiqqer-locations-locator-filter');

                if (!empty($filterData[$idHash])) {
                    $FilterControl->setFilterData($filterData[$idHash]);
                }
            }
        }

        // Display
        $listClass   = 'quiqqer-locations-locator-element-30';
        $mapClass    = 'quiqqer-locations-locator-element-70';
        $filterClass = 'quiqqer-locations-locator-element-30';

        if (!$showMap) {
            $listClass = 'quiqqer-locations-locator-element-100';
        }

        if (!$showList) {
            $mapClass = 'quiqqer-locations-locator-element-100';
        }

        // CSS
        $mapHeight = (int)$Site->getAttribute('quiqqer.locations.locator.map.height_px');

        if (empty($mapHeight) || $mapHeight <= 0) {
            $mapHeight = 750;
        }

        // display template
        switch ($this->getAttribute('display')) {
            case 'filterSlideOut':
                $template = \dirname(__FILE__).'/Locator.FilterSlideOut.html';
                $css      = \dirname(__FILE__).'/Locator.FilterSlideOut.css';
                break;

            case 'resultListBottom':
                $template = \dirname(__FILE__).'/Locator.ResultListBottom.html';
                $css      = \dirname(__FILE__).'/Locator.ResultListBottom.css';
                break;

            case '3Columns':
            default:
                $template = \dirname(__FILE__).'/Locator.3Columns.html';
                $css      = \dirname(__FILE__).'/Locator.3Columns.css';
        }

        $Engine->assign([
            'showList'               => $showList,
            'showMap'                => $showMap,
            'showUserLocationFilter' => $showUserLocationFilter,
            'listClass'              => $listClass,
            'mapClass'               => $mapClass,
            'listSettings'           => $listSettings,
            'mapSettings'            => $mapSettings,
            'userLocationSettings'   => $userLocationSettings,
            'showCountrySelect'      => $showCountrySelect,
            'countries'              => $this->getCountries(),
            'userCountry'            => $userCountry,
            'mapHeight'              => $mapHeight,
            'filterControls'         => $filterControls,
            'filterClass'            => $filterClass,
            'radiusOptions'          => $radiusOptions,
            'radiusDefault'          => $radiusDefault
        ]);

        $this->addCSSFile($css);

        return $Engine->fetch($template);
    }

    /**
     * Get list of locations that are shown on the map on load
     *
     * @return array|false - Array with location ID, coordinates and map icon; false if no locations should be shown initially
     */
    protected function getMapLocationList()
    {
        try {
            $showOnEmptySearchResults = LocatorHandler::getLocatorSetting('showOnEmptySearchResults');
        } catch (\Exception $Exception) {
            QUI\System\Log::writeException($Exception);

            return false;
        }

        $locationIds = $this->getAttribute('locationIds');

        if ($locationIds === false) {
            return false;
        }

        if (empty($locationIds)) {
            if ($showOnEmptySearchResults === LocatorHandler::MAP_EMPTY_RESULTS_SHOW_ALL) {
                return false;
            } else {
                return [];
            }
        }

        try {
            $locations = QUI\Locations\Locator::getAllLocationPoints([
                'allowedIds' => $locationIds
            ]);
        } catch (\Exception $Exception) {
            QUI\System\Log::writeException($Exception);

            return false;
        }

        if (empty($locations)) {
            if ($showOnEmptySearchResults === LocatorHandler::MAP_EMPTY_RESULTS_SHOW_ALL) {
                return false;
            } else {
                return [];
            }
        }

        return $locations;
    }

    /**
     * Get content of LocatorList of locations that are shown on the list on load
     *
     * @return string|false - HTML content of LocatorListResult control; false if no locations should be shown initially
     */
    protected function getListContent()
    {
        $locationIds = $this->getAttribute('locationIds');

//        if (empty($locationIds)) {
//            return false;
//        }

        try {
            $GeoCenter = QUI\Locations\Handler\LocationSearch::getGeoCenterByRequest();

            $ListContent = new LocatorListResult([
                'Site'          => $this->getSite(),
                'Center'        => $GeoCenter ? $GeoCenter->getPosition() : false,
                'radius'        => $this->getAttribute('radius'),
                'locationIds'   => $this->getAttribute('locationIds'),
                'locationImage' => $this->getAttribute('locationImageList')
            ]);

            return $ListContent->create();
        } catch (\Exception $Exception) {
            QUI\System\Log::writeException($Exception);

            return false;
        }
    }

    /**
     * Get list of countries from quiqqer/countries
     *
     * @return array
     * @throws QUI\Exception
     */
    protected function getCountries()
    {
        $dir = QUI::getPackage('quiqqer/countries')->getDir();

        QUI::getAjax(); // init Ajax

        require_once $dir.'ajax/getCountries.php';

        $result = QUI::$Ajax->callRequestFunction(
            'package_quiqqer_countries_ajax_getCountries',
            [
                'lang' => $this->getSite()->getProject()->getLang()
            ]
        );

        return $result['result'];
    }

    /**
     * Return the current site
     *
     * @return mixed|QUI\Projects\Site
     *
     * @throws QUI\Exception
     */
    protected function getSite()
    {
        if ($this->getAttribute('Site')) {
            return $this->getAttribute('Site');
        }

        return QUI::getRewrite()->getSite();
    }
}
