<?php

namespace QUI\Locations\Controls\Search;

use QUI;
use QUI\Countries\Manager as Countries;
use QUI\Utils\Security\Orthos;
use QUI\LocationCategories\Handler as LocationCategories;
use QUI\Locations\Handler\LocationSearch as LocationSearchHandler;

/**
 * Class LocationSearch
 *
 * Search locations by zip code and category
 */
class LocationSearch extends QUI\Control
{
    /**
     * constructor
     *
     * @param array $attributes
     */
    public function __construct($attributes = [])
    {
        $this->setAttributes([
            'showRadiusInfo' => true,
            'submitAction'   => LocationSearchHandler::SUBMIT_ACTION_LIST,
            'useCategories'  => true // use category list as filter
        ]);

        $this->addCSSClass('quiqqer-locations-search-locations');
        $this->addCSSFile(dirname(__FILE__).'/LocationSearch.css');
        $this->setJavaScriptControl('package/quiqqer/locations/bin/frontend/controls/Search/LocationSearch');

        parent::__construct($attributes);
    }

    /**
     * (non-PHPdoc)
     *
     * @throws QUI\Exception
     * @see \QUI\Control::create()
     *
     */
    public function getBody()
    {
        // Determine submit action
        $Site    = $this->getSite();
        $country = false;

        try {
            $action  = $Site->getAttribute('quiqqer.locations.locationsearch.action');
            $country = $Site->getAttribute('quiqqer.locations.locationsearch.country');
        } catch (\Exception $Exception) {
            QUI\System\Log::writeException($Exception);
        }

        if (empty($action)) {
            $action = LocationSearchHandler::SUBMIT_ACTION_LIST;
        }

        // Determine country
        $UserCountry = QUI::getUserBySession()->getCountry();
        $userCountry = false;

        if (empty($UserCountry)) {
            $userCountry = $country;
        }

        if (empty($userCountry)) {
            if (Countries::existsCountryCode(QUI::getLocale()->getCurrent())) {
                $userCountry = \mb_strtoupper(QUI::getLocale()->getCurrent());
            }
        }

        if (empty($userCountry)) {
            $userCountry = 'DE';
        }

        $this->setJavaScriptControlOption('country', $userCountry);

        // Determine action site
        $ActionSite = false;

        if ($action === LocationSearchHandler::SUBMIT_ACTION_MAP) {
            try {
                $ActionSite = QUI\Locations\Locator\Handler::getLocatorSite();
            } catch (\Exception $Exception) {
                QUI\System\Log::writeException($Exception);
            }
        }

        if (!$ActionSite) {
            $ActionSite = LocationSearchHandler::getLocationSearchSite();
        }

        if (!$ActionSite) {
            QUI\System\Log::addWarning(
                'Location search cannot be displayed -> No sites of type "'.LocationSearchHandler::SITE_TYPE_LOCATIONSEARCH.'"'
                .' or "'.QUI\Locations\Locator\Handler::SITE_TYPE_LOCATOR.'" found.'
            );

            return '';
        }

        $Engine        = QUI::getTemplateManager()->getEngine();
        $useCategories = false;

        if ($this->getAttribute('useCategories') && QUI::getPackageManager()->isInstalled('quiqqer/locationcategories')) {
            $Engine->assign('categories', $this->getCategories());
            $useCategories = true;
        }

        $Engine->assign([
            'showRadiusInfo' => $this->getAttribute('showRadiusInfo'),
            'radius'         => LocationSearchHandler::getSearchRadius($Site),
            'ActionSite'     => $ActionSite,
            'useCategories'  => $useCategories,
            'catId'          => empty($_REQUEST['cat_id']) ? false : (int)$_REQUEST['cat_id'],
            'searchString'   => empty($_REQUEST['search_string']) ? false : Orthos::clear($_REQUEST['search_string'])
        ]);

        return $Engine->fetch(\dirname(__FILE__).'/LocationSearch.html');
    }

    /**
     * Get category list for
     *
     * @return array
     * @throws QUI\Exception
     * @throws QUI\LocationCategories\Exception
     */
    protected function getCategories()
    {
        $categories = [
            'all' => [
                'id'            => 'all',
                'title'         => QUI::getLocale()->get(
                    'quiqqer/locations',
                    'controls.LocationSearch.category.all'
                ),
                'subcategories' => []
            ]
        ];

        $categoryList = LocationCategories::getCategoryTitleList();

        foreach ($categoryList as $category) {
            $Category       = LocationCategories::getCategory($category['id']);
            $ParentCategory = $Category->getParentCategory();

            if (!$ParentCategory) {
                continue;
            }

            if (empty($categories[$ParentCategory->getId()])) {
                $categories[$ParentCategory->getId()] = [
                    'id'            => $ParentCategory->getId(),
                    'title'         => $ParentCategory->getTitle(),
                    'subcategories' => []
                ];
            }

            $categories[$ParentCategory->getId()]['subcategories'][] = $category;
        }

        return $categories;
    }

    /**
     * Return the current site
     *
     * @return mixed|QUI\Projects\Site
     *
     * @throws QUI\Exception
     */
    protected function getSite()
    {
        if ($this->getAttribute('Site')) {
            return $this->getAttribute('Site');
        }

        return QUI::getRewrite()->getSite();
    }
}
