<?php

namespace QUI\Locations;

use QUI\Utils\StringHelper;
use QUI\Utils\Text\XML;
use QUI;
use QUI\Utils\System\File as QUIFile;
use QUI\Package\Package;
use QUI\Locations\Handler\Attributes as AttributesHandler;
use QUI\Locations\Handler\Locations as LocationsHandler;
use QUI\Locations\Handler\LocationAPIs as LocationsApisHandler;
use QUI\Locations\Locator\Handler as Locator;
use QUI\Projects\Media\Utils as MediaUtils;
use QUI\Locations\Location\Location;
use QUI\Cache\Manager as QUICacheManager;

/**
 * Class Events
 *
 * quiqqer/locations QUIQQER Events
 *
 * @package QUI\Locations
 */
class Events
{
    /**
     * Contains Location IDs of all Locations that are
     * deleted in this runtime (so the media folder can be deleted too)
     *
     * @var array
     */
    public static $deleteLocations = [];

    /**
     * QUIQQER Event: onPackageSetup
     *
     * @param \QUI\Package\Package $Package
     * @return void
     */
    public static function onPackageSetup(Package $Package)
    {
        if ($Package->getName() !== 'quiqqer/locations') {
            return;
        }

//        QUI\Countries\Setup::setup();

        self::readLocationsXmlFiles();
        self::readLocationApiXmlFiles();
    }

    /**
     * QUIQQER Event: onMediaDeleteBegin
     *
     * @param QUI\Projects\Media\Item $Item - The media item that is to be deleted
     *
     * @throws Exception
     */
    public static function onMediaDeleteBegin(QUI\Projects\Media\Item $Item)
    {
        if (!MediaUtils::isFolder($Item)) {
            return;
        }

        /** @var QUI\Projects\Media\Folder $Item */
        $locations = LocationsHandler::getLocationsWithAttributeValue(
            'imagesMediaFolder',
            $Item->getUrl()
        );

        if (empty($locations)) {
            return;
        }

        /** @var QUI\Locations\Location\Location $Location */
        $Location = $locations[0];

        if (in_array($Location->getId(), self::$deleteLocations)) {
            return;
        }

        throw new Exception([
            'quiqqer/locations',
            'exception.events.onMediaDeleteBegin.cannot.delete.locations.folder',
            [
                'id'         => $Item->getId(),
                'locationId' => $Location->getId()
            ]
        ]);
    }

    /**
     * Reads all locations.xml files from installed modules and registers
     * location attributes
     *
     * @return void
     * @throws Exception
     */
    protected static function readLocationsXmlFiles()
    {
        if (!defined('OPT_DIR')) {
            throw new Exception(
                'exception.events.packages.dir.not.found'
            );
        }

        $locationAttributes = [];
        $packages           = QUIFile::readDir(OPT_DIR);
        $L                  = QUI::getLocale();

        // read all locations.xml files and parse locale attributes
        foreach ($packages as $package) {
            if ($package === 'composer') {
                continue;
            }

            $packageDir = OPT_DIR.'/'.$package;
            $list       = QUIFile::readDir($packageDir);

            foreach ($list as $sub) {
                if (!is_dir($packageDir.'/'.$sub)) {
                    continue;
                }

                $xmlFile = $packageDir.'/'.$sub.'/locations.xml';

                if (!file_exists($xmlFile)) {
                    continue;
                }

                $Dom        = XML::getDomFromXml($xmlFile);
                $entries    = $Dom->getElementsByTagName('quiqqer');
                $attributes = $entries->item(0)
                    ->getElementsByTagName('locations')
                    ->item(0)
                    ->getElementsByTagName('attributes')
                    ->item(0)
                    ->getElementsByTagName('attribute');

                if (!$attributes->length) {
                    return;
                }

                for ($i = 0; $i < $attributes->length; $i++) {
                    $attributeData = [];
                    $Attribute     = $attributes->item($i);

                    if ($Attribute->hasAttribute('class')) {
                        $attributeData['attribute_class'] = $Attribute->getAttribute('class');
                    }

                    $name = $Attribute->getElementsByTagName('name');

                    if (!$name->length) {
                        QUI::getMessagesHandler()->addAttention(
                            $L->get(
                                'quiqqer/locations',
                                'error.package.setup.locations.xml.missing.name',
                                [
                                    'package' => $packageDir.'/'.$sub
                                ]
                            )
                        );

                        continue;
                    }

                    $attributeData['name'] = $name->item(0)->nodeValue;

                    $dbColumnType = $Attribute->getElementsByTagName('db_column_type');

                    if ($dbColumnType->length) {
                        $attributeData['columnType'] = $dbColumnType->item(0)->nodeValue;
                    }

                    $priority = $Attribute->getElementsByTagName('priority');

                    if ($priority->length) {
                        $attributeData['priority'] = $priority->item(0)->nodeValue;
                    }

                    $title = $Attribute->getElementsByTagName('title');

                    if (!$title->length) {
                        QUI::getMessagesHandler()->addAttention(
                            $L->get(
                                'quiqqer/locations',
                                'error.package.setup.locations.xml.missing.title',
                                [
                                    'package' => $package.'/'.$sub
                                ]
                            )
                        );

                        continue;
                    }

                    $attributeData['title']       = self::readLocaleDataFromXmlNode($title->item(0));
                    $attributeData['description'] = null;

                    $description = $Attribute->getElementsByTagName('description');

                    if ($description->length) {
                        $attributeData['description'] = self::readLocaleDataFromXmlNode($description->item(0));
                    }

                    $attributeData['source_package'] = $package.'/'.$sub;
                    $locationAttributes[]            = $attributeData;
                }
            }
        }

        try {
            self::processLocationAttributes($locationAttributes);
        } catch (Exception $Exception) {
            QUI\System\Log::addError(
                self::class.' :: readLocationsXmlFiles -> '.$Exception->getMessage()
            );

            QUI::getMessagesHandler()->addError(
                $L->get(
                    'quiqqer/locations',
                    'error.package.setup.locations.error',
                    [
                        'error' => $Exception->getMessage()
                    ]
                )
            );
        }
    }

    /**
     * Reads all locationsapi.xml files from installed modules and registers
     * api classes and controls
     *
     * @return void
     * @throws Exception
     */
    protected static function readLocationApiXmlFiles()
    {
        if (!defined('OPT_DIR')) {
            throw new Exception(
                'exception.events.packages.dir.not.found'
            );
        }

        $locationApis = [];
        $packages     = QUIFile::readDir(OPT_DIR);
        $L            = QUI::getLocale();

        // read all locations.xml files and parse locale attributes
        foreach ($packages as $package) {
            if ($package === 'composer') {
                continue;
            }

            $packageDir = OPT_DIR.'/'.$package;
            $list       = QUIFile::readDir($packageDir);

            foreach ($list as $sub) {
                if (!is_dir($packageDir.'/'.$sub)) {
                    continue;
                }

                $xmlFile = $packageDir.'/'.$sub.'/locationsapi.xml';

                if (!file_exists($xmlFile)) {
                    continue;
                }

                $data = [
                    'title'       => '',
                    'description' => null,
                    'jsControl'   => null,
                    'execMethod'  => null
                ];

                $Dom     = XML::getDomFromXml($xmlFile);
                $entries = $Dom->getElementsByTagName('quiqqer');
                $ApiData = $entries->item(0)
                    ->getElementsByTagName('locations')
                    ->item(0)
                    ->getElementsByTagName('api')
                    ->item(0);

                $TitleNode = $ApiData->getElementsByTagName('title');

                if (!$TitleNode->length) {
                    QUI::getMessagesHandler()->addAttention(
                        $L->get(
                            'quiqqer/locations',
                            'error.package.setup.locationsapi.xml.missing.title',
                            [
                                'package' => $packageDir.'/'.$sub
                            ]
                        )
                    );

                    continue;
                }

                $data['title'] = json_encode(self::readLocaleDataFromXmlNode($TitleNode->item(0)));

                $DescNode = $ApiData->getElementsByTagName('description');

                if ($DescNode->length) {
                    $data['description'] = json_encode(self::readLocaleDataFromXmlNode($DescNode->item(0)));
                }

                $JsControlNode = $ApiData->getElementsByTagName('jsControl');

                if ($JsControlNode->length) {
                    $JsControlNode = $JsControlNode->item(0);

                    if ($JsControlNode->hasAttribute('path')) {
                        $data['jsControl'] = $JsControlNode->getAttribute('path');
                    }
                }

                $ExecNode = $ApiData->getElementsByTagName('exec');

                if (!$ExecNode->length) {
                    QUI::getMessagesHandler()->addAttention(
                        $L->get(
                            'quiqqer/locations',
                            'error.package.setup.locationsapi.xml.missing.exec',
                            [
                                'package' => $packageDir.'/'.$sub
                            ]
                        )
                    );

                    continue;
                }

                $ExecNode = $ExecNode->item(0);

                if (!$ExecNode->hasAttribute('class')) {
                    QUI::getMessagesHandler()->addAttention(
                        $L->get(
                            'quiqqer/locations',
                            'error.package.setup.locationsapi.xml.missing.exec',
                            [
                                'package' => $packageDir.'/'.$sub
                            ]
                        )
                    );

                    continue;
                }

                $data['execMethod']     = $ExecNode->getAttribute('class');
                $data['source_package'] = $package.'/'.$sub;
                $locationApis[]         = $data;
            }
        }

        try {
            $DB = QUI::getDataBase();

            foreach ($locationApis as $locationApi) {
                $result = $DB->fetch([
                    'select' => [
                        'id'
                    ],
                    'from'   => LocationsApisHandler::getLocationsApiTable(),
                    'where'  => [
                        'source_package' => $locationApi['source_package']
                    ]
                ]);

                if (empty($result)) {
                    $locationApi['execTime'] = json_encode([
                        'enabled' => false,
                        'time'    => [
                            'minute'    => '*',
                            'hour'      => '*',
                            'day'       => '*',
                            'month'     => '*',
                            'dayOfWeek' => '*'
                        ]
                    ]);

                    $DB->insert(
                        LocationsApisHandler::getLocationsApiTable(),
                        $locationApi
                    );
                } else {
                    unset($locationApi['source_package']);

                    $DB->update(
                        LocationsApisHandler::getLocationsApiTable(),
                        $locationApi,
                        [
                            'id' => $result[0]['id']
                        ]
                    );
                }
            }
        } catch (Exception $Exception) {
            QUI\System\Log::addError(
                self::class.' :: readLocationApiXmlFiles -> '.$Exception->getMessage()
            );

            QUI::getMessagesHandler()->addError(
                $L->get(
                    'quiqqer/locations',
                    'error.package.setup.locationsapis.error',
                    [
                        'error' => $Exception->getMessage()
                    ]
                )
            );
        }
    }

    /**
     * Process locale attributes
     *
     * @param array $attributes
     *
     * @throws Exception
     */
    protected static function processLocationAttributes($attributes)
    {
        $DB            = QUI::getDataBase();
        $tblSearch     = LocationsHandler::getLocationsSearchTable();
        $tblAttributes = AttributesHandler::getLocationAttributesTable();

        foreach ($attributes as $attribute) {
            $exists = true;

            try {
                AttributesHandler::getLocationAttribute($attribute['name']);
            } catch (Exception $Exception) {
                $exists = false;
            }

            $column     = $attribute['name'];
            $columnType = 'TEXT';

            if (isset($attribute['columnType'])) {
                $columnType = $attribute['columnType'];
            }

            if ($DB->table()->existColumnInTable($tblSearch, $column)) {
                if (!$exists) {
                    throw new Exception([
                        'quiqqer/locations',
                        'exception.package.setup.duplicate.search.column',
                        [
                            'column' => $column
                        ]
                    ]);
                }

                $Statement = $DB->getPDO()->prepare(
                    'ALTER TABLE '.$tblSearch.' MODIFY COLUMN '.$column
                    .' '.$columnType.' NULL;'
                );

                $Statement->execute();
            } else {
                $DB->table()->addColumn($tblSearch, [
                    $column => $columnType.' NULL'
                ]);
            }

            if ($exists) {
                $DB->update(
                    $tblAttributes,
                    [
                        'priority'        => $attribute['priority'],
                        'title'           => json_encode($attribute['title']),
                        'description'     => json_encode($attribute['description']),
                        'source_package'  => $attribute['source_package'],
                        'attribute_class' => isset($attribute['attribute_class']) ? $attribute['attribute_class'] : null
                    ],
                    [
                        'name' => $attribute['name']
                    ]
                );

                continue;
            }

            $DB->insert(
                $tblAttributes,
                [
                    'name'            => $attribute['name'],
                    'priority'        => $attribute['priority'],
                    'title'           => json_encode($attribute['title']),
                    'description'     => json_encode($attribute['description']),
                    'source_package'  => $attribute['source_package'],
                    'attribute_class' => isset($attribute['attribute_class']) ? $attribute['attribute_class'] : null
                ]
            );
        }
    }

    /**
     * Read locale data from locale DOMElement
     *
     * @param \DOMElement $Node - Node that CONTAINTS a <locale>-Node
     * @return array
     */
    protected static function readLocaleDataFromXmlNode(\DOMElement $Node)
    {
        $LocaleNode = $Node->getElementsByTagName('locale')->item(0);

        return [
            'group' => $LocaleNode->getAttribute('group'),
            'var'   => $LocaleNode->getAttribute('var')
        ];
    }

    /**
     * quiqqer/quiqqer: onSiteSave
     *
     * @param \QUI\Projects\Site $Site
     * @return void
     */
    public static function onSiteSave(\QUI\Projects\Site $Site)
    {
        if ($Site->getAttribute('type') !== Locator::SITE_TYPE_LOCATOR) {
            return;
        }

        if (!$Site->getAttribute('active')) {
            return;
        }

        // Register path for locations
        try {
            $url = $Site->getLocation();
            $url = StringHelper::strReplaceFromEnd(QUI\Rewrite::URL_DEFAULT_SUFFIX, '', $url);

            QUI::getRewrite()->registerPath($url.'/*', $Site);
        } catch (\Exception $Exception) {
            QUI\System\Log::writeException($Exception);
        }
    }
}
