<?php

namespace QUI\Locations\Handler;

use Cron\CronExpression;
use QUI;
use QUI\Locations\Utils;

/**
 * Handles multiple location APIs
 *
 * @package QUI\Locations
 */
class LocationAPIs
{
    const TBL_LOCATIONS_APIS = 'quiqqer_locations_apis';

    /**
     * Runtime cache for API list
     *
     * @var array
     */
    protected static $list = null;

    /**
     * Get list of all registered locations apis
     *
     * @return array
     */
    public static function getList()
    {
        if (!is_null(self::$list)) {
            return self::$list;
        }

        $list = array();

        $result = QUI::getDataBase()->fetch(array(
            'from' => self::getLocationsApiTable()
        ));

        foreach ($result as $row) {
            $row['title'] = Utils::getLocaleFromArray(json_decode($row['title'], true));

            if (!empty($row['description'])) {
                $row['description'] = Utils::getLocaleFromArray(json_decode($row['description'], true));
            }

            $list[] = $row;
        }

        self::$list = $list;

        return $list;
    }

    /**
     * Execute specific API
     *
     * @param int $id - API ID
     * @return void
     *
     * @throws QUI\Locations\Exception
     */
    public static function execute($id)
    {
        $list = self::getList();
        $data = false;

        foreach ($list as $entry) {
            if ($entry['id'] == $id) {
                $data = $entry;
                break;
            }
        }

        if (!$data) {
            return;
        }

        $execMethod = $data['execMethod'];

        try {
            call_user_func($execMethod);

            QUI::getDataBase()->update(
                self::getLocationsApiTable(),
                array(
                    'lastExec' => time()
                ),
                array(
                    'id' => $data['id']
                )
            );
        } catch (\Exception $Exception) {
            QUI\System\Log::addError(
                self::class . ' :: execute() -> ' . $Exception->getMessage()
            );

            throw new QUI\Locations\Exception(array(
                'quiqqer/locations',
                'exceptions.handler.locationapis.execute.error',
                array(
                    'apiTitle' => $data['title'],
                    'apiId'    => $data['id']
                )
            ));
        }
    }

    /**
     * Update execution time for API
     *
     * @param int $id - Location API ID
     * @param array $execTime
     *
     * @throws QUI\Locations\Exception
     */
    public static function updateExecTime($id, $execTime)
    {
        if (!is_array($execTime)
            || !isset($execTime['enabled'])
            || !isset($execTime['time'])
        ) {
            throw new QUI\Locations\Exception(array(
                'quiqqer/locations',
                'exception.handler.locationapis.exectime.wrong.format'
            ));
        }

        // check time format
        $time = $execTime['time'];

        if (!isset($time['minute']) ||
            !isset($time['hour']) ||
            !isset($time['day']) ||
            !isset($time['month']) ||
            !isset($time['dayOfWeek'])
        ) {
            throw new QUI\Locations\Exception(array(
                'quiqqer/locations',
                'exception.handler.locationapis.exectime.wrong.format'
            ));
        }

        try {
            CronExpression::factory(
                $time['minute'] . " "
                . $time['hour'] . " "
                . $time['day'] . " "
                . $time['month'] . " "
                . $time['dayOfWeek']
            );
        } catch (\Exception $Exception) {
            throw new QUI\Locations\Exception(array(
                'quiqqer/locations',
                'exception.handler.locationapis.exectime.wrong.format'
            ));
        }

        QUI::getDataBase()->update(
            self::getLocationsApiTable(),
            array(
                'execTime' => json_encode($execTime)
            ),
            array(
                'id' => $id
            )
        );
    }

    /**
     * Get table name of location table
     *
     * @return string
     */
    public static function getLocationsApiTable()
    {
        return QUI::getDBTableName(self::TBL_LOCATIONS_APIS);
    }
}
