<?php

namespace QUI\Locations\Location;

use QUI;
use QUI\LocationCategories\Handler as LocationCategories;
use QUI\Locations\Handler\Locations as LocationsHandler;
use QUI\Locations\Handler\Attributes as AttributesHandler;
use QUI\Locations\Exception;
use QUI\Locations\Locator\Handler as LocatorHandler;
use QUI\Permissions\Permission;
use QUI\Lock\Locker;
use QUI\Locations\Events as LocationsEvents;
use QUI\Projects\Media\Utils as QUIMediaUtils;

/**
 * Class Location
 *
 * Represents a single location
 *
 * @package QUI\Locations
 */
class Location
{
    /**
     * Location specific permissions
     */
    const PERMISSION_CREATE     = 'quiqqer.locations.create';
    const PERMISSION_EDIT       = 'quiqqer.locations.edit';
    const PERMISSION_FORCE_EDIT = 'quiqqer.locations.force_edit_location';
    const PERMISSION_DELETE     = 'quiqqer.locations.delete';

    /**
     * Location ID
     *
     * @var int
     */
    protected $id = null;

    /**
     * Location attributes runtime cache
     *
     * @var array
     */
    protected $locationAttributes = [];

    /**
     * Active status of this Location
     *
     * @var bool
     */
    protected $active = false;

    /**
     * Location View runtime cache
     *
     * @var View
     */
    protected $View = null;

    /**
     * Information about creation and last edit of location data
     *
     * @var array
     */
    protected $editInfo = [];

    /**
     * Location constructor.
     *
     * @param int $id - Location ID
     * @throws QUI\Locations\Exception
     */
    public function __construct($id)
    {
        $result = QUI::getDataBase()->fetch([
            'from'  => LocationsHandler::getLocationsTable(),
            'where' => [
                'id' => $id
            ]
        ]);

        if (empty($result)) {
            throw new Exception([
                'quiqqer/locations',
                'exception.location.not.found',
                [
                    'locationId' => $id
                ]
            ], 404);
        }

        $locationData   = $result[0];
        $this->id       = (int)$locationData['id'];
        $this->active   = $locationData['active'] ? true : false;
        $this->editInfo = [
            'c_date' => $locationData['c_date'],
            'c_user' => $locationData['c_user'],
            'e_date' => $locationData['e_date'],
            'e_user' => $locationData['e_user']
        ];


        $this->setLocationAttributes(json_decode($locationData['attributes'], true));
    }

    /**
     * Set multiple location attributes
     *
     * @param array $attributes
     * @return void
     */
    public function setLocationAttributes($attributes)
    {
        foreach ($attributes as $attribute => $value) {
            $this->setLocationAttribute($attribute, $value);
        }
    }

    /**
     * Set value to a location attribute
     *
     * @param string $attribute
     * @param mixed $value
     * @return void
     *
     * @throws QUI\Locations\Exception
     */
    public function setLocationAttribute($attribute, $value)
    {
        // cannot change id
        if ($attribute == 'id') {
            return;
        }

        try {
            $Attribute = $this->getLocationAttribute($attribute);
        } catch (QUI\Locations\Exception $Exception) {
            if ($Exception->getCode() === 404) {
                return;
            }

            throw $Exception;
        }

        try {
            $Attribute->setValue($value);
        } catch (QUI\Locations\Exception $Exception) {
            throw new QUI\Locations\Exception([
                'quiqqer/locations',
                'exception.location.attribute.error',
                [
                    'attribute' => $Attribute->getTitle(),
                    'error'     => $Exception->getMessage()
                ]
            ]);
        }
    }

    /**
     * Get location attribute
     *
     * @param string $attribute - attribute name
     * @return QUI\Locations\Attributes\AbstractAttribute
     * @throws Exception
     */
    public function getLocationAttribute($attribute)
    {
        if (isset($this->locationAttributes[$attribute])) {
            return $this->locationAttributes[$attribute];
        }

        $attributeClass = AttributesHandler::getLocationAttributeClass($attribute);

        $this->locationAttributes[$attribute] = new $attributeClass($attribute);

        return $this->locationAttributes[$attribute];
    }

    /**
     * Get all location attributes
     *
     * @return array
     */
    public function getLocationAttributes()
    {
        return $this->locationAttributes;
    }

    /**
     * Get Location ID
     *
     * @return int
     */
    public function getId()
    {
        return $this->id;
    }

    /**
     * Activate Location
     *
     * @return void
     *
     * @throws QUI\Locations\Exception
     */
    public function activate()
    {
        if ($this->active) {
            return;
        }

        if ($this->isLocked()) {
            if (!Permission::hasPermission($this::PERMISSION_FORCE_EDIT)) {
                throw new QUI\Locations\Exception([
                    'quiqqer/locations',
                    'exception.location.locked'
                ]);
            }
        } else {
            Permission::checkPermission($this::PERMISSION_EDIT);
        }

        // update locations table
        QUI::getDataBase()->update(
            LocationsHandler::getLocationsTable(),
            [
                'active' => 1
            ],
            [
                'id' => $this->id
            ]
        );

        // update search table
        QUI::getDataBase()->update(
            LocationsHandler::getLocationsSearchTable(),
            [
                'active' => 1
            ],
            [
                'id' => $this->id
            ]
        );

        $this->active = true;

        QUI::getEvents()->fireEvent('onLocationActivate', [$this]);
    }

    /**
     * Deactivate Location
     *
     * @return void
     *
     * @throws QUI\Locations\Exception
     */
    public function deactivate()
    {
        if (!$this->active) {
            return;
        }

        if ($this->isLocked()) {
            if (!Permission::hasPermission($this::PERMISSION_FORCE_EDIT)) {
                throw new QUI\Locations\Exception([
                    'quiqqer/locations',
                    'exception.location.locked'
                ]);
            }
        } else {
            Permission::checkPermission($this::PERMISSION_EDIT);
        }

        // update locations table
        QUI::getDataBase()->update(
            LocationsHandler::getLocationsTable(),
            [
                'active' => 0
            ],
            [
                'id' => $this->id
            ]
        );

        // update search table
        QUI::getDataBase()->update(
            LocationsHandler::getLocationsSearchTable(),
            [
                'active' => 0
            ],
            [
                'id' => $this->id
            ]
        );

        $this->active = false;

        QUI::getEvents()->fireEvent('onLocationDeactivate', [$this]);
    }

    /**
     * Check if this Location is active
     *
     * @return bool
     */
    public function isActive()
    {
        return $this->active;
    }

    /**
     * Save location data to database
     *
     * @param bool $triggerEvents (optional) - Trigger save events [default: true]
     * @return void
     *
     * @throws QUI\Locations\Exception
     * @throws QUI\Permissions\Exception
     * @throws QUI\Database\Exception
     * @throws QUI\Exception
     */
    public function save(bool $triggerEvents = true)
    {
        if ($this->isLocked()) {
            if (!Permission::hasPermission($this::PERMISSION_FORCE_EDIT)) {
                throw new QUI\Locations\Exception([
                    'quiqqer/locations',
                    'exception.location.locked'
                ]);
            }
        } else {
            Permission::checkPermission($this::PERMISSION_EDIT);
        }

        if ($triggerEvents) {
            QUI::getEvents()->fireEvent('onLocationSaveBefore', [$this]);
        }

        $DB               = QUI::getDataBase();
        $attributes       = [];
        $searchAttributes = [];
        $SearchLocale     = new QUI\Locale();
        $langs            = QUI::availableLanguages();

        /** @var QUI\Locations\Attributes\AbstractAttribute $Attribute */
        foreach ($this->locationAttributes as $Attribute) {
            $attributes[$Attribute->getName()] = $Attribute->getValue();

            foreach ($langs as $lang) {
                $SearchLocale->setCurrent($lang);

                if (!isset($searchAttributes[$lang])) {
                    $searchAttributes[$lang] = [];
                }

                $searchAttributes[$lang][$Attribute->getName()] = $Attribute->getSearchCacheValue($SearchLocale);
            }
        }

        // update locations table
        $eDate = time();
        $eUser = QUI::getUsers()->getSystemUser()->getId();

        $DB->update(
            LocationsHandler::getLocationsTable(),
            [
                'attributes' => json_encode($attributes),
                'e_date'     => $eDate,
                'e_user'     => $eUser
            ],
            [
                'id' => $this->id
            ]
        );

        $this->editInfo['e_date'] = $eDate;
        $this->editInfo['e_user'] = $eUser;

        // determine LATITUDE and LONGITUDE
        $lat              = null;
        $lng              = null;
        $Coordinates      = $this->getLocationAttribute('coordinates');
        $coordinatesValue = $Coordinates->getValue();

        if (!empty($coordinatesValue)) {
            $coordinates = \json_decode($Coordinates->getValue(), true);
            $coordinates = $coordinates['coordinates'];

            if (!empty($coordinates['lat'])) {
                $lat = (float)$coordinates['lat'];
            }

            if (!empty($coordinates['lng'])) {
                $lng = (float)$coordinates['lng'];
            }
        }

        // update locations search table
        foreach ($searchAttributes as $lang => $attributes) {
            $DB->update(
                LocationsHandler::getLocationsSearchTable(),
                array_merge($attributes, ['lat' => $lat, 'lng' => $lng]),
                [
                    'id'   => $this->id,
                    'lang' => $lang
                ]
            );
        }

        if ($triggerEvents) {
            QUI::getEvents()->fireEvent('onLocationSave', [$this]);
        }
    }

    /**
     * Delete this location
     *
     * @return void
     *
     * @throws QUI\Locations\Exception
     */
    public function delete()
    {
        if ($this->isLocked()) {
            throw new QUI\Locations\Exception([
                'quiqqer/locations',
                'exception.location.locked'
            ]);
        }

        Permission::checkPermission($this::PERMISSION_DELETE);

        // add location id to event array
        LocationsEvents::$deleteLocations[] = $this->id;

        QUI::getEvents()->fireEvent('onLocationDelete', [$this]);

        // delete from locations table
        QUI::getDataBase()->delete(
            LocationsHandler::getLocationsTable(),
            [
                'id' => $this->id
            ]
        );

        // delete from locations search table
        QUI::getDataBase()->delete(
            LocationsHandler::getLocationsSearchTable(),
            [
                'id' => $this->id
            ]
        );
    }

    /**
     * Get Location View
     *
     * @return View
     */
    public function getView()
    {
        if (!is_null($this->View)) {
            return $this->View;
        }

        $this->View = new View($this);

        return $this->View;
    }

    /**
     * @return array
     */
    public function getEditInfo(): array
    {
        return $this->editInfo;
    }

    /**
     * Check if this location is currently being edited by another user
     *
     * @return bool
     */
    public function isLocked()
    {
        return Locker::isLocked(QUI::getPackage('quiqqer/locations'), 'location_'.$this->id);
    }

    /**
     * Get Location URL
     *
     * @param QUI\Projects\Project $Project (optional) - [default: get Project via Rewrite]
     * @return string|false - URL or false if not reachable via URL
     */
    public function getUrl(QUI\Projects\Project $Project = null)
    {
        if (QUI::getPackageManager()->isInstalled('quiqqer/locationcategories')) {
            $url = LocationCategories::getLocationUrl($this);

            if (!empty($url)) {
                return $url;
            }
        }

        $LocatorSite = LocatorHandler::getLocatorSite($Project);

        if (empty($LocatorSite)) {
            return false;
        }

        try {
            return $LocatorSite->getUrlRewrittenWithHost([], [
                'id' => $this->getId()
            ]);
        } catch (\Exception $Exception) {
            QUI\System\Log::writeException($Exception);
            return false;
        }
    }

    /**
     * Get map icon for this location
     *
     * @return QUI\Projects\Media\Image|false
     */
    public function getMapIcon()
    {
        // First: Get map icon saved in location
        try {
            $MapIconAttribute = $this->getLocationAttribute('mapIcon');

            if (!empty($MapIconAttribute->getValue())) {
                return QUIMediaUtils::getImageByUrl($MapIconAttribute->getValue());
            }
        } catch (\Exception $Exception) {
            QUI\System\Log::writeException($Exception);
        }

        // Second: Try to get map icon of associated location category
        if (QUI::getPackageManager()->isInstalled('quiqqer/locationcategories')) {
            try {
                $LocationCategory = LocationCategories::getCategoryByLocation($this);

                if ($LocationCategory) {
                    $CategoryMapIcon = $LocationCategory->getMapIcon();

                    if ($CategoryMapIcon !== false) {
                        return $CategoryMapIcon;
                    }
                }
            } catch (\Exception $Exception) {
                QUI\System\Log::writeException($Exception);
            }
        }

        // Third: Get default map icon by module setting
        // @todo

        return false;
    }

    /**
     * Location data as array
     *
     * @return array
     */
    public function toArray()
    {
        $createUser = QUI::getUsers()->get($this->editInfo['c_user'])->getUsername();
        $editUser   = QUI::getUsers()->get($this->editInfo['e_user'])->getUsername();

        $data = [
            'id'         => $this->id,
            'active'     => $this->active,
            'c_date'     => date("Y-m-d H:i:s", $this->editInfo['c_date']),
            'c_user'     => $createUser.' ('.$this->editInfo['c_user'].')',
            'e_date'     => date("Y-m-d H:i:s", $this->editInfo['e_date']),
            'e_user'     => $editUser.' ('.$this->editInfo['e_user'].')',
            'attributes' => []
        ];

        /** @var QUI\Locations\Attributes\AbstractAttribute $Attribute */
        foreach ($this->locationAttributes as $Attribute) {
            $data['attributes'][$Attribute->getName()] = $Attribute->getValue();
        }

        return $data;
    }
}
