<?php

namespace QUI\Locations\Location;

use QUI;

/**
 * Class View
 *
 * Represents a single location view object with frontend data
 *
 * @package QUI\Locations
 */
class View
{
    /**
     * Location model class
     *
     * @var Location
     */
    protected $Location = null;

    /**
     * View constructor.
     *
     * @param Location $Location - The location object that is represented in a view state
     */
    public function __construct(Location $Location)
    {
        $this->Location = $Location;
    }

    /**
     * Get Location ID
     *
     * @return int
     */
    public function getId()
    {
        return $this->Location->getId();
    }

    /**
     * Get view data of location attribute
     *
     * @param string $attribute - attribute name
     * @return mixed
     */
    public function getAttribute($attribute)
    {
        return $this->Location->getLocationAttribute($attribute)->getViewValue();
    }

    /**
     * Get Location URL
     *
     * @param QUI\Projects\Project $Project (optional) - [default: get Project via Rewrite]
     * @return string|false - URL or false if not reachable via URL
     */
    public function getUrl(QUI\Projects\Project $Project = null)
    {
        return $this->Location->getUrl($Project);
    }

    /**
     * Get address data as individual address text lines
     *
     * @return string[]
     */
    public function getAddressLines()
    {
        $data = [];

        /** @var QUI\Locations\Attributes\AbstractAttribute $Attribute */
        foreach ($this->Location->getLocationAttributes() as $Attribute) {
            $data[$Attribute->getName()] = $Attribute->getViewValue();
        }

        $addressLines = [];

        if (!empty($data['street'])) {
            $streetLine = $data['street'];

            if (!empty($data['houseNumber'])) {
                $streetLine .= ' '.$data['houseNumber'];
            }

            $addressLines[] = $streetLine;
        }

        if (!empty($data['city'])) {
            $cityLine = $data['city'];

            if (!empty($data['zipCode'])) {
                $cityLine = $data['zipCode'].' '.$cityLine;
            }

            $addressLines[] = $cityLine;
        }

        if (!empty($data['country'])) {
            $addressLines[] = $data['country'];
        }

        return $addressLines;
    }

    /**
     * Location data as array
     *
     * @return array
     */
    public function toArray()
    {
        $attributes = [];

        /** @var QUI\Locations\Attributes\AbstractAttribute $Attribute */
        foreach ($this->Location->getLocationAttributes() as $Attribute) {
            $attributes[$Attribute->getName()] = $Attribute->getViewValue();
        }

        // determine if open
        $attributes['openNow'] = !empty($attributes['openingHours']['openNow'])
                                 || !empty($attributes['specialOpeningHours']['openNow']);

        $attributes['id']           = $this->getId();
        $attributes['active']       = $this->Location->isActive();
        $attributes['addressLines'] = $this->getAddressLines();
        $attributes['mainImageUrl'] = false;

        if (!empty($attributes['mainImage'])) {
            try {
                $attributes['mainImageUrl'] = $this->Location->getLocationAttribute('mainImage')->getValue();
            } catch (\Exception $Exception) {
                QUI\System\Log::writeException($Exception);
            }
        } else {
            $DefaultImage = QUI\Locations\Handler\Locations::getLocationDefaultImage();

            if (!empty($DefaultImage)) {
                $attributes['mainImageUrl'] = $DefaultImage->getUrl();
            }
        }

        return $attributes;
    }
}
